#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "mp3db.h"
#include "util.h"
#include "display.h"
#include "search.h"
#include "preferences.h"
#include "serialize.h"
#include "xmms2.h"

GtkWidget *searchdialog = NULL;
GtkWidget *aboutdialog = NULL;
GtkWidget *prefdialog = NULL;
GtkWidget *savedialog = NULL;
GtkWidget *loaddialog = NULL;

extern GtkWidget *mainwindow;
extern Preferences *prefs;
extern int build_active, progress_update;
extern int max_songs,max_artists,max_albums;
extern MP3File **song_db;
extern gboolean search_select;

void
on_search1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(!searchdialog) {
		searchdialog=create_searchdialog();
		init_tv_search();
	}
	gtk_widget_show(searchdialog);
}

void
on_random1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	int rnd=(int) ((float)max_songs*rand()/(RAND_MAX+1.0));
	if (rnd>=0) {
		search_select=TRUE;
		select_item("tv_artist",ART_IDX,song_db[rnd]->Artist);
		select_item("tv_album",ALB_IDX,song_db[rnd]->Album);
		select_item("tv_song",COL_IDX,rnd);
		search_select=FALSE;
		//printf(" auto enqueue '%s'\n",song_db[rnd]->songname);
		//add_to_playlist(song_db[rnd]->filename);
	}
}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(!prefdialog){
		prefdialog=create_preferences(); 
		set_preferences(prefdialog);
	}
	gtk_widget_show(prefdialog);
}


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_widget_destroy(mainwindow);
}


void
on_play1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	xmms2_cmd(XCMD_PLAY);
}


void
on_pause1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	xmms2_cmd(XCMD_PAUSE);
}


void
on_prev1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	xmms2_cmd(XCMD_PREV);
}


void
on_next1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	xmms2_cmd(XCMD_NEXT);
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

	if(!aboutdialog) aboutdialog=create_aboutdialog();
	gtk_widget_show(aboutdialog);
}

void
on_initialize1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	build_db();
}


void
on_do_search_activate                  (GtkButton       *button,
                                        gpointer         user_data)
{
	do_search();
}


void
on_search_hide_activate                (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(searchdialog);
	// free tv_search  and list ??
	searchdialog=NULL;
	status_default();
}


void
on_mainwindow_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
	if (prefs->restore) {
	  gchar *cfg=get_home_cfg(".mp3browser.db");
	  save_db(cfg);
	  g_free(cfg);
	}
  	close_xmms_session();
	save_preferences();
	// force (possible) threads to exit 
	build_active=0; progress_update = -1;
	gtk_main_quit();
}


void
on_searchdialog_destroy                (GtkObject       *object,
                                        gpointer         user_data)
{
	searchdialog=NULL;
	status_default();
}


void
on_preferences_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	prefdialog=NULL;
}


void
on_pref_ok_activate                    (GtkButton       *button,
                                        gpointer         user_data)
{
	parse_preferences(prefdialog);
	gtk_widget_destroy(prefdialog);
	prefdialog=NULL;
	save_preferences();
	update_gui();
}


void
on_pref_hide_activate                  (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(prefdialog);
	prefdialog=NULL;
}


void
on_load_database1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkResponseType i;
	if(!loaddialog) loaddialog=create_loaddialog(); 
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (loaddialog), "/tmp");
	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (loaddialog),"mp3browser.db");
	if ((i=gtk_dialog_run (GTK_DIALOG (loaddialog))) == GTK_RESPONSE_OK || 
			i==GTK_RESPONSE_ACCEPT)
	{
		char *filename;
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (loaddialog));
		gtk_widget_destroy(loaddialog);
		status_msg("loading Database...");
		loaddialog=NULL;

		while (gtk_events_pending ()) gtk_main_iteration();
		load_db(filename);
		update_list_artist();
		status_default();
		g_free (filename);
	}
	loaddialog=NULL;
}


void
on_save_database1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkResponseType i;
	if(!savedialog) savedialog=create_savedialog(); 
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (savedialog), "/tmp");
	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (savedialog), "mp3browser.db");
	if ((i=gtk_dialog_run (GTK_DIALOG (savedialog))) == GTK_RESPONSE_OK || 
			i==GTK_RESPONSE_ACCEPT)
	{
		char *filename;
		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (savedialog));
		gtk_widget_destroy(savedialog);
		savedialog=NULL;
		status_msg("saving Database...");
		while (gtk_events_pending ()) gtk_main_iteration();
		save_db (filename);
		status_default();
		g_free (filename);
	}
	savedialog=NULL;
}


void
on_load_default_database1_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar *cfg=get_home_cfg(".mp3browser.db");
	load_db(cfg);
	g_free(cfg);
	update_list_artist();
	status_default();
}

static void get_sel_foreach(GtkTreeModel *tm, GtkTreePath *tp, GtkTreeIter *iter, gpointer data) {
	int idx;
	char *name;
	gtk_tree_model_get(tm, iter, COL_IDX, &idx, -1);
	name = song_db[idx]->filename;
	if (name) {
		gchar *uri = g_filename_to_uri (name, NULL, NULL);
		if (uri) {
			add_to_playlist (uri);
			g_free (uri);
			(*((guint*)data))++;
		}
	}
}

void
on_button_xmms_activated               (GtkButton       *button,
                                        gpointer         user_data)
{
	guint added=0;
	update_gui();
	GtkTreeSelection *ts = NULL;
	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(lookup_widget(mainwindow,"tv_song")));
	gtk_tree_selection_selected_foreach(ts, get_sel_foreach, &added);
	//printf("added %i songs\n",added);
	//TODO: jump to (last-in-playlist - added) !
	//xmms2_cmd(XCMD_PLAY);
}

void
on_now_playing_activated               (GtkButton       *button,
                                        gpointer         user_data)
{
	select_now_playing();
}

void
on_xmms2_toggled                       (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if(gtk_toggle_button_get_active (togglebutton)) {
		init_xmms_session();
	} else {
		close_xmms_session();
	}
	update_gui();
}

void
on_toggle_rnd_toggled                  (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	prefs->add_random=gtk_toggle_button_get_active (togglebutton);
	update_gui();
}

void cb_drag_data_received(GtkWidget          *widget,
		           GdkDragContext     *context,
		           gint                x,
		           gint                y,
		           GtkSelectionData   *data,
		           guint               info,
		           guint               time)
{
	if ((data->length >= 0) && (data->format == 8)) {
		gint n = 0, m = 0;
		gchar buffer[384], *uri_conv;
		/* seperate each url in the list */
		while (n < strlen((gchar *)data->data)) {
			if (((guchar *)data->data)[n] == '\n') {
				buffer[m] = '\0';
				m = 0;
				/* decode %XX escape sequences to their corresponding chars: */
				uri_conv = decode_string(buffer);
				add_to_playlist (uri_conv);
				g_free(uri_conv);
			} else if (((guchar *)data->data)[n] > 31) {
				buffer[m] = ((guchar *)data->data)[n];
				if (m < 382) m++;
			}
			n++;
		}
		gtk_drag_finish(context, TRUE, FALSE, time);
	} else {
		gtk_drag_finish(context, FALSE, FALSE, time);
	}
}



