#define _GNU_SOURCE

#define INCLUDE_DND

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "mp3db.h"
#include "display.h"
#include "preferences.h"

extern GtkWidget *mainwindow;

extern Artist **artist_db;
extern Album **album_db;
extern MP3File **song_db;

extern Preferences *prefs;
extern SongStatistics **stat_db;

extern int max_songs,max_artists,max_albums;
extern int want_verbose;
extern gboolean search_select;

int cur_songs,cur_artists,cur_albums;

GtkListStore *ls_artist;
GtkListStore *ls_album;
GtkListStore *ls_song;


/* prev. selected items */
int p_artist = -2;
int p_album = -2;


int progress_update=-1;
extern int build_active;

/*
 * Drag and Drop (Song)
 */

static void
on_tm_dnd_get_uri_foreach(GtkTreeModel *tm, GtkTreePath *tp,
			  GtkTreeIter *iter, gpointer data)
{
	int idx;
	GString *filelist = (GString *)data;
	char *name;

	gtk_tree_model_get(tm, iter, COL_IDX, &idx, -1);
	name = song_db[idx]->filename;
	if (name)
	{
		gchar *uri = g_filename_to_uri (name, NULL, NULL);
		if (uri)
		{
			g_string_append_printf (filelist, "%s\n", uri);
			g_free (uri);
		}
	}
}

static void
on_tm_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp,
			   GtkTreeIter *iter, gpointer data)
{
	int idx;
	GString *filelist = (GString *)data;
	char *name;

	gtk_tree_model_get(tm, iter, COL_IDX, &idx, -1);
	name = song_db[idx]->filename;
	if (name) g_string_append_printf (filelist, "file:%s\n", name);
}


static void tm_drag_data_get (GtkWidget       *widget,
			      GdkDragContext  *context,
			      GtkSelectionData *data,
			      guint            info,
			      guint            time,
			      gpointer         user_data)
{
    GtkTreeSelection *ts = NULL;
    GString *reply = g_string_sized_new (2047);


    if((data) && (ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget))))
    {
	switch (info)
	{
	case DND_TEXT_URI_LIST:
	    gtk_tree_selection_selected_foreach(ts,
				    on_tm_dnd_get_uri_foreach, reply);
	    break;
	case DND_TEXT_PLAIN:
	    gtk_tree_selection_selected_foreach(ts,
				    on_tm_dnd_get_file_foreach, reply);
	    break;
	default:
	    g_warning ("Programming error: ta_drag_data_get received unknown info type (%d)\n", info);
	    break;
	}
    }
    gtk_selection_data_set(data, data->target, 8, (guchar*) reply->str, reply->len);
    g_string_free (reply, TRUE);
}


/*
 * Drag and Drop (Album)
 */

static void
on_ta_dnd_get_uri_foreach(GtkTreeModel *tm, GtkTreePath *tp,
			  GtkTreeIter *iter, gpointer data)
{
	int idx;
	GString *filelist = (GString *)data;
	char *name=NULL;

	gtk_tree_model_get(tm, iter, ALB_IDX, &idx, -1);
	if (idx>=0) name = album_db[idx]->playlist;
	if (name)
	{
		gchar *uri = g_filename_to_uri (name, NULL, NULL);
		if (uri)
		{
			g_string_append_printf (filelist, "%s\n", uri);
			g_free (uri);
		}
	}
}

static void
on_ta_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp,
			   GtkTreeIter *iter, gpointer data)
{
	int idx;
	GString *filelist = (GString *)data;
	char *name=NULL;

	gtk_tree_model_get(tm, iter, ALB_IDX, &idx, -1);
	if (idx>=0) name = album_db[idx]->playlist;
	if (name) g_string_append_printf (filelist, "file:%s\n", name);
}


static void ta_drag_data_get (GtkWidget       *widget,
			      GdkDragContext  *context,
			      GtkSelectionData *data,
			      guint            info,
			      guint            time,
			      gpointer         user_data)
{
    GtkTreeSelection *ts = NULL;
    GString *reply = g_string_sized_new (2047);


    if((data) && (ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget))))
    {
	switch (info)
	{
	case DND_TEXT_URI_LIST:
	    gtk_tree_selection_selected_foreach(ts,
				    on_ta_dnd_get_uri_foreach, reply);
	    break;
	case DND_TEXT_PLAIN:
	    gtk_tree_selection_selected_foreach(ts,
				    on_ta_dnd_get_file_foreach, reply);
	    break;
	default:
	    g_warning ("Programming error: ta_drag_data_get received unknown info type (%d)\n", info);
	    break;
	}
	/* if no playlist - fall back to selecting all songs */
	if (reply->len == 0) {
		ts=gtk_tree_view_get_selection (GTK_TREE_VIEW(lookup_widget(mainwindow,"tv_song")));
		gtk_tree_selection_select_all(ts);
		switch (info)
		{
		case DND_TEXT_URI_LIST:
		    gtk_tree_selection_selected_foreach(ts, on_tm_dnd_get_uri_foreach, reply);
		    break;
		case DND_TEXT_PLAIN:
		    gtk_tree_selection_selected_foreach(ts, on_tm_dnd_get_file_foreach, reply);
		    break;
		default:
		    g_warning ("Programming error: ta_drag_data_get received unknown info type (%d)\n", info);
		    break;
		}
		gtk_tree_selection_unselect_all(ts);
	}
    }
    gtk_selection_data_set(data, data->target, 8, (guchar*) reply->str, reply->len);
    g_string_free (reply, TRUE);
}


/*
 * TreeView compare function
 * for Sorting the Lists
 */

gint sort_compare_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer userdata) {
	gint sortcol = GPOINTER_TO_INT(userdata); 
	gint compcol=COL_TITLE;
	gint ret = 0; 
	/* for search list sort */
	if (sortcol == SORTID_ARTIST){ compcol=0; sortcol=SORTID_TITLE;}
	if (sortcol == SORTID_ALBUM){ compcol=1; sortcol=SORTID_TITLE;}
	if (sortcol == SORTID_SONG){ compcol=2; sortcol=SORTID_TITLE;}
	if (sortcol == SORTID_INDEX){ compcol=COL_INDEX;}

	switch (sortcol) { 
		case SORTID_TITLE: {
					  char *name1, *name2; 
					  gtk_tree_model_get(model, a, compcol, &name1, -1); 
					  gtk_tree_model_get(model, b, compcol, &name2, -1); 
					  if (name1 == NULL || name2 == NULL) { 
						  if (name1 == NULL && name2 == NULL) break; 
						  ret = (name1 == NULL) ? -1 : 1; 
					  } 
					  else  ret = strcmp(name1,name2); 
					  
					  free(name1); free(name2); 
				  } 
		case SORTID_ALBUMALL: {
					  char *name1, *name2; 
					  gtk_tree_model_get(model, a, compcol, &name1, -1); 
					  gtk_tree_model_get(model, b, compcol, &name2, -1); 
					  if (name1 == NULL || name2 == NULL) { 
						  if (name1 == NULL && name2 == NULL) break; 
						  ret = (name1 == NULL) ? -1 : 1; 
					  } 
					  else  {
						  if (strcmp(name1,"All")==0) ret=-1;
						  else if (strcmp(name2,"All")==0) ret=1;
						  else ret = strcmp(name1,name2); 
					  }
					  
					  free(name1); free(name2); 
				  } 
				  break; 
		case SORTID_INDEX: 
		case SORTID_STATS: 
				  { 
					  int idx1, idx2; 
					  char *idxs1, *idxs2;
					  gtk_tree_model_get(model, a, sortcol, &idxs1, -1); 
					  gtk_tree_model_get(model, b, sortcol, &idxs2, -1); 

					  if (idxs1 == NULL || idxs2 == NULL) { 
						  if (idxs1 == NULL && idxs2 == NULL) break; 
						  ret = (idxs1 == NULL) ? -1 : 1; 
					  } 
					  idx1=atoi(idxs1); idx2=atoi(idxs2);
					  if (idx1 != idx2) { ret = (idx1 > idx2) ? 1 : -1; } 
					  free(idxs1); free(idxs2); 
				   }
				  break; 
		default: g_return_val_if_reached(0); 
	} 
	return ret; 
}

/*
 * update 'now playing' button status
 */

void xmms_buttons(gboolean active) {
	GtkWidget *w=lookup_widget(mainwindow,"button9");
	gtk_widget_set_sensitive(w,active);
}

/* 
 * select items in mainwindow 
 */

void scrollbar_jump (char *scrollwidget,int pos,int max) {
	float f,e;
	if (max <1) return;

	GtkScrolledWindow *sc = GTK_SCROLLED_WINDOW(lookup_widget(mainwindow,scrollwidget));
	GtkAdjustment *adj = gtk_scrolled_window_get_vadjustment(sc);
	
	f=((float)pos/(float)max)*(adj->upper-adj->lower);
	e=(((float)pos+1.0)/(float)max)*(adj->upper-adj->lower);

	gtk_adjustment_clamp_page(adj,f,e);
//	gtk_adjustment_set_value(adj,f);
//	gtk_adjustment_changed(adj);
}

void select_item(char *treeview, int col, int cmp) 
{
	GtkTreeView *tv;
	GtkTreeIter iter;
	GtkTreeModel *tm;
	int cnt=0;

	if (build_active || progress_update >=0) return;

	tv=GTK_TREE_VIEW(lookup_widget(mainwindow,treeview));
	tm=gtk_tree_view_get_model (tv);
	gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(tv));
	gtk_tree_model_get_iter_first(tm,&iter);
	do {
		int idx;
		gtk_tree_model_get(tm, &iter, col, &idx, -1);
		if (idx == cmp) { 
			gtk_tree_selection_select_iter(gtk_tree_view_get_selection(tv),&iter);

			// update lists -> update scrollbars -> update scrollbar range
			while (gtk_events_pending ()) gtk_main_iteration();

			if (strcmp(treeview,"tv_artist")== 0) 
				scrollbar_jump ("scrolledwindow1",cnt,cur_artists);
			if (strcmp(treeview,"tv_album")== 0) 
				scrollbar_jump ("scrolledwindow3",cnt,cur_albums);
			if (strcmp(treeview,"tv_song")== 0) 
				scrollbar_jump ("scrolledwindow2",cnt,cur_songs);
			break;
		}
		cnt++;
	} while (gtk_tree_model_iter_next(tm,&iter));
}

/*
 * 'selection changed' callbacks
 */

gboolean func_artist_sel (GtkTreeSelection *selection,
				GtkTreeModel     *model,
				GtkTreePath      *path,
				gboolean          path_currently_selected,
				gpointer          userdata)
{
	GtkTreeIter iter;
	if (gtk_tree_model_get_iter(model, &iter, path))
	{
		gchar *name;
		gtk_tree_model_get(model, &iter, 0, &name, -1);
		if (!path_currently_selected) 
		{
			int artist=get_artist_id(name);
			if (artist!=p_artist) {
				p_artist=artist;
				p_album=-2;
				update_list_album(artist);
				status_top ();
			}
		}
		g_free(name);
	}
	return TRUE; /* allow selection state to change */
}

gboolean func_album_sel (GtkTreeSelection *selection,
				GtkTreeModel     *model,
				GtkTreePath      *path,
				gboolean          path_currently_selected,
				gpointer          userdata)
{
	GtkTreeIter iter;
	if (gtk_tree_model_get_iter(model, &iter, path))
	{
		int album;
		gtk_tree_model_get(model, &iter, ALB_IDX, &album, -1);
		if (!path_currently_selected) 
		{
			if (album!=p_album) {
				p_album=album;
				update_list_song(p_artist,p_album);
				// if (p_album>=0) status_bar ("bar_album", album_db[p_album]->album);
				status_top();
				update_album_artwork(p_artist,p_album);
			}
		}
	}
	return TRUE; /* allow selection state to change */
}

/***********************************
 * Initialisation of the Lists
 */

void init_tv_artist (void) {
	GtkTreeView *tv_artist;
	GtkCellRenderer *renderer;
	GtkTreeSelection *selection;
	GtkTreeViewColumn *col;
	GtkTreeSortable *sortable;

	ls_artist = gtk_list_store_new (ART_LAST,G_TYPE_STRING,G_TYPE_INT);
	sortable = GTK_TREE_SORTABLE(ls_artist);
	gtk_tree_sortable_set_sort_func(sortable, SORTID_TITLE, sort_compare_func, 
			GINT_TO_POINTER(SORTID_TITLE), NULL);
	gtk_tree_sortable_set_sort_column_id(sortable, SORTID_TITLE, GTK_SORT_ASCENDING);

	tv_artist=GTK_TREE_VIEW(lookup_widget(mainwindow,"tv_artist"));
	gtk_tree_view_set_model (tv_artist, GTK_TREE_MODEL (ls_artist));
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (tv_artist), GTK_SELECTION_BROWSE);

	selection = gtk_tree_view_get_selection(tv_artist);
        gtk_tree_selection_set_select_function(selection, func_artist_sel, NULL, NULL);


	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Artist");

	/* pack tree view column into tree view */
	gtk_tree_view_append_column(tv_artist, col);

        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", ART_NAME);
	
}

void init_tv_album (void) {
	GtkTreeView *tv_album;
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;
	GtkTreeSelection *selection;
	GtkTreeSortable *sortable;

	ls_album = gtk_list_store_new (ALB_LAST,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_INT);

	sortable = GTK_TREE_SORTABLE(ls_album);
	gtk_tree_sortable_set_sort_func(sortable, SORTID_TITLE, sort_compare_func, 
			GINT_TO_POINTER(SORTID_ALBUMALL), NULL);
	gtk_tree_sortable_set_sort_column_id(sortable, SORTID_TITLE, GTK_SORT_ASCENDING);

	tv_album=GTK_TREE_VIEW(lookup_widget(mainwindow,"tv_album"));
	gtk_tree_view_set_model (tv_album, GTK_TREE_MODEL (ls_album));
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (tv_album), GTK_SELECTION_BROWSE);

	gtk_drag_source_set(GTK_WIDGET(tv_album),GDK_BUTTON1_MASK,
			drag_types, TGNR(drag_types),GDK_ACTION_COPY);
	gtk_signal_connect (GTK_OBJECT(tv_album), "drag_data_get", G_CALLBACK(ta_drag_data_get),NULL);

	selection = gtk_tree_view_get_selection(tv_album);
        gtk_tree_selection_set_select_function(selection, func_album_sel, NULL, NULL);

	col = gtk_tree_view_column_new();
	gtk_tree_view_append_column(tv_album, col);
	gtk_tree_view_column_set_sizing(col,GTK_TREE_VIEW_COLUMN_AUTOSIZE);
        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", ALB_NAME);

	col = gtk_tree_view_column_new();
	gtk_tree_view_append_column(tv_album, col);
	gtk_tree_view_column_set_sizing(col,GTK_TREE_VIEW_COLUMN_AUTOSIZE);
        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", ALB_NUM);
}

void init_tv_song (void) {
	GtkTreeView *tv_song;
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;
	GtkTreeSortable *sortable;
//	GtkTooltips *tooltips;
//	tooltips = gtk_tooltips_new ();

	ls_song = gtk_list_store_new (COL_LAST, G_TYPE_STRING,
			G_TYPE_STRING, G_TYPE_STRING,
			G_TYPE_STRING, G_TYPE_INT);

	sortable = GTK_TREE_SORTABLE(ls_song);
	gtk_tree_sortable_set_sort_func(sortable, COL_TITLE, sort_compare_func, 
			GINT_TO_POINTER(SORTID_TITLE), NULL);
	gtk_tree_sortable_set_sort_func(sortable, COL_INDEX, sort_compare_func,
			GINT_TO_POINTER(SORTID_INDEX), NULL);
	gtk_tree_sortable_set_sort_func(sortable, COL_STATS, sort_compare_func,
			GINT_TO_POINTER(SORTID_STATS), NULL);
	gtk_tree_sortable_set_sort_column_id(sortable, SORTID_INDEX, GTK_SORT_ASCENDING);

	tv_song=GTK_TREE_VIEW(lookup_widget(mainwindow,"tv_song"));
	gtk_tree_view_set_model (tv_song, GTK_TREE_MODEL (ls_song));
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (tv_song), GTK_SELECTION_MULTIPLE);

	gtk_drag_source_set(GTK_WIDGET(tv_song),GDK_BUTTON1_MASK,
			drag_types, TGNR(drag_types),GDK_ACTION_COPY);
	gtk_signal_connect (GTK_OBJECT(tv_song), "drag_data_get", G_CALLBACK(tm_drag_data_get),NULL);


	/* column 1 */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Title");
	gtk_tree_view_append_column(tv_song, col);
	gtk_tree_view_column_set_resizable(col,TRUE);
	gtk_tree_view_column_set_reorderable(col,TRUE);
	gtk_tree_view_column_set_sizing(col,GTK_TREE_VIEW_COLUMN_AUTOSIZE);

        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_TITLE);
	gtk_tree_view_column_set_sort_column_id(col,SORTID_TITLE);

	/* column 2 */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Index");
	gtk_tree_view_append_column(tv_song, col);
	gtk_tree_view_column_set_reorderable(col,TRUE);
	gtk_tree_view_column_set_sizing(col,GTK_TREE_VIEW_COLUMN_AUTOSIZE);

        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_INDEX);
	gtk_tree_view_column_set_sort_column_id(col,SORTID_INDEX);

	/* column 3 */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Length");
	gtk_tree_view_append_column(tv_song, col);
	gtk_tree_view_column_set_reorderable(col,TRUE);
	gtk_tree_view_column_set_sizing(col,GTK_TREE_VIEW_COLUMN_AUTOSIZE);

        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_LENGTH);
	//gtk_tree_view_column_set_sort_column_id(col,SORTID_LENGTH);

	/* column 4 */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Statistics");
	gtk_tree_view_append_column(tv_song, col);
	gtk_tree_view_column_set_reorderable(col,TRUE);
	gtk_tree_view_column_set_sizing(col,GTK_TREE_VIEW_COLUMN_AUTOSIZE);

        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", COL_STATS);
	//gtk_tree_view_column_set_sort_column_id(col,SORTID_STATS);
}

void init_treeviews (void) {
	init_tv_artist();
	init_tv_album();
	init_tv_song();
}

/*
 * update displayed data 
 */

//static gpointer update_progress_thread (gpointer gp)
gboolean update_progress_thread (gpointer gp)
{
	if (progress_update<0) return (0);
	status_printf("updating Artist list... (%i/%i entries)",progress_update,max_artists);
	return(1);
}

void update_list_artist(void) {

	GtkTreeView *tv_artist;
	GtkTreeIter iter;
	Artist **last;
	int idx=0;
	progress_update=0;

	status_printf("updating Artist list... (%i/%i entries)",0,max_artists);
	while (gtk_events_pending ()) gtk_main_iteration();
//	progress_thread = g_thread_create (update_progress_thread, NULL, FALSE, NULL);
	g_timeout_add(500,update_progress_thread,NULL);

	last=artist_db;
	tv_artist=GTK_TREE_VIEW(lookup_widget(mainwindow,"tv_artist"));
	gtk_list_store_clear(ls_artist);

	g_object_ref(ls_artist); /* Make sure the model stays with us after the tree view unrefs it */
	gtk_tree_view_set_model (tv_artist, NULL ); /* speed up insertion process */

	cur_artists=0;
	while (last && *last) {
		gtk_list_store_append (ls_artist, &iter);
		gtk_list_store_set (ls_artist, &iter,
				ART_NAME,(*last)->artist,
				ART_IDX,idx,
				-1);
		cur_artists++;
		last++;idx++;
		progress_update++;
		while (gtk_events_pending ()) gtk_main_iteration();
	}
	progress_update=-1;
	gtk_tree_view_set_model (tv_artist, GTK_TREE_MODEL (ls_artist));
	g_object_unref(ls_artist); 
	status_default();
}


void update_list_album(int Artist) {
	GtkTreeIter iter,all_iter,alb_iter;
	GtkTreeView *tv_album;
	Album **last;
	int idx=0;
	gboolean alb_iter_set=FALSE;
	char *num;

	tv_album=GTK_TREE_VIEW(lookup_widget(mainwindow,"tv_album"));
	gtk_list_store_clear(ls_album);
	gtk_list_store_clear(ls_song);

	gtk_list_store_append (ls_album, &all_iter);
	asprintf(&num,"(%i)", count_songs_by_artist(Artist));
	gtk_list_store_set (ls_album, &all_iter,
			ALB_NAME,"All",
			ALB_NUM,num,
			ALB_IDX,-1,
			-1);
	free(num);

	g_object_ref(ls_album); /* Make sure the model stays with us after the tree view unrefs it */
	gtk_tree_view_set_model (tv_album, NULL ); /* speed up insertion process */

	last=album_db;
	cur_albums=0;
	while (last && *last) {
		if ((*last)->Artist != Artist) {last++;idx++; continue;}
		if ((*last)->tracks == 0) (*last)->tracks=count_songs_by_album(Artist,idx);
		asprintf(&num,"(%i)%s",(*last)->tracks,(*last)->playlist?"":" no index");
		gtk_list_store_append (ls_album, &iter);
		if (!alb_iter_set) {alb_iter=iter; alb_iter_set=TRUE;}
		gtk_list_store_set (ls_album, &iter,
				ALB_NAME,(*last)->album,
				ALB_NUM,num,
				ALB_IDX,idx,
				-1);
		cur_albums++;
		last++;idx++;
		free(num);
	}
	gtk_tree_view_set_model (tv_album, GTK_TREE_MODEL (ls_album));
	g_object_unref(ls_album); 

	if (prefs->auto_select_album == 2 && alb_iter_set) all_iter=alb_iter;

	if (!search_select && prefs->auto_select_album )
		gtk_tree_selection_select_iter(gtk_tree_view_get_selection(tv_album),&all_iter);
}


void update_list_song(int Artist, int Album) {
	GtkTreeIter iter;
	GtkTreeView *tv_song;
	MP3File **last;
	int idx=0;

	last=song_db;
	gtk_list_store_clear(ls_song);
	tv_song=GTK_TREE_VIEW(lookup_widget(mainwindow,"tv_song"));
	g_object_ref(ls_song); /* Make sure the model stays with us after the tree view unrefs it */
	gtk_tree_view_set_model (tv_song, NULL ); /* speed up insertion process */

	cur_songs=0;
	while (last && *last) {
		char *index, *length, *stats;
		int len;

		if ((*last)->Artist != Artist ||
		    (Album!=-1 && (*last)->Album != Album)) {last++;idx++; continue;}

		asprintf(&index,"%i / %i",(*last)->track_nr,(*last)->tracks);
		len=(*last)->tracklen/1000;	
		asprintf(&length,"%.0i%s%02i:%02i",len/3600,len>3600?":":"",(len%3600)/60,len%60);
		asprintf(&stats,"-");

		gtk_list_store_append (ls_song, &iter);
		gtk_list_store_set (ls_song, &iter,
				COL_TITLE,(*last)->songname,
				COL_INDEX,index,
				COL_LENGTH,length,
				COL_STATS,stats,
				COL_IDX,idx,
				-1);

		cur_songs++;
		last++;idx++;
		free(index); free(length); free(stats);
	}
	gtk_tree_view_set_model (tv_song, GTK_TREE_MODEL (ls_song));
	g_object_unref(ls_song); 
}

void update_album_artwork(int Artist, int Album) {
	GtkImage *album_image;
	GdkPixbuf *pixbuf;

	album_image=GTK_IMAGE(lookup_widget(mainwindow,"album_image"));

	if (Album<0 || !album_db[Album] || !album_db[Album]->coverimage) {
		gtk_image_set_from_pixbuf (album_image,NULL);
		return;
	}

	if(want_verbose) printf("loading image: %s\n",album_db[Album]->coverimage);

	pixbuf = gdk_pixbuf_new_from_file_at_scale (album_db[Album]->coverimage,160,160,TRUE,NULL);
	if (pixbuf) {
		gtk_image_set_from_pixbuf (album_image,pixbuf);
		g_object_unref(pixbuf); 
	} else { 
		printf("image load failed \n");
		status_msg("image load failed.");
	}

}

/*
 * Status Bar
 */

void status_msg (char *msg) {
	gchar buf[PATH_MAX];
	guint context = 1;
	GtkWidget *statusbar1;
	statusbar1=lookup_widget(mainwindow,"statusbar1");

	snprintf(buf, PATH_MAX, "%s", msg);
	gtk_statusbar_pop(GTK_STATUSBAR(statusbar1), context);
	gtk_statusbar_push(GTK_STATUSBAR(statusbar1), context,  buf);
}


void status_printf (const char *format, ...) {
	va_list arglist;
	char text[PATH_MAX];

	va_start(arglist, format);
	vsnprintf(text, PATH_MAX, format, arglist);
	va_end(arglist);
	text[PATH_MAX -1] =0;
	
	status_msg(text);
}


void status_bar (char *barname, char *msg) {
	gchar buf[PATH_MAX];
	guint context = 1;
	GtkWidget *bar;
	bar=lookup_widget(mainwindow,barname);

	snprintf(buf, PATH_MAX, "%s", msg);
	gtk_statusbar_pop(GTK_STATUSBAR(bar), context);
	gtk_statusbar_push(GTK_STATUSBAR(bar), context,  buf);
}

void status_top (void) {
	char *text;
	if (p_album>0) asprintf(&text,"%s - %s",artist_db[p_artist]->artist, album_db[p_album]->album);
	else asprintf(&text,"%s",artist_db[p_artist]->artist);
	status_bar ("bar_artist", text);
	free(text);
}

void status_default (void) {
	status_printf("Database: %i Songs by %i Artists (%i Albums).",
			max_songs,max_artists,max_albums);
}

