#define _GNU_SOURCE


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "mp3db.h"
#include "display.h"
#include "preferences.h"

#include <string.h>
#include <sys/types.h>
#include <dirent.h>

Artist **artist_db=NULL;
Album **album_db=NULL;
MP3File **song_db=NULL;

int build_active;

extern Preferences *prefs;
extern SongStatistics **stat_db;

int max_songs,max_artists,max_albums;
int totalfiles=0;

/*
 * Database content functions
 */

void *free_artist (void *d) {
	Artist  *a=(Artist*)d;
	if (!a) return (NULL);
	if (a->artist) free(a->artist);
	free(a);
	return(NULL);
}

void *free_album (void *d) {
	Album  *a=(Album*)d;
	if (!a) return (NULL);
	if (a->album) free(a->album);
	if (a->coverimage) free(a->coverimage);
	if (a->playlist) free(a->playlist);
	free(a);
	return(NULL);
}

void *free_mf (void *d) {
	MP3File  *mf=(MP3File*)d;
	if (!mf) return (NULL);
	if (mf->songname) free(mf->songname);
	if (mf->filename) free(mf->filename);
	free(mf);
	return(NULL);
}

void clear_db_artist (void) {
	Artist **last;
	last=artist_db;
	while (last && *last) { free_artist(*last); last++; }
	free(artist_db);
	artist_db=NULL;
	max_artists=0;
}

void clear_db_album (void) {
	Album **last;
	last=album_db;
	while (last && *last) { free_album(*last); last++; }
	free(album_db);
	album_db=NULL;
	max_albums=0;
}

void clear_db_song (void) {
	MP3File **last;
	last=song_db;
	while (last && *last) { free_mf(*last); last++; }
	free(song_db);
	song_db=NULL;
	max_songs=0;
}

void clear_db (void) {
	clear_db_artist();
	clear_db_album();
	clear_db_song();
}

MP3File *dup_mf (MP3File *mf) {
	MP3File *new;
	char *tmp;
	new=(MP3File *) calloc(1,sizeof(MP3File));
	if (mf->songname) new->songname=strdup(mf->songname);
	// process fn : replace  '//' by '/', strip newline,...
	// is this bad that we do it in the old  instead of the new string??
	if ((tmp=strchr(mf->filename,'\n'))) *tmp=0;
	if ((tmp=strchr(mf->filename,'\r'))) *tmp=0;

	while ((tmp=strstr(mf->filename,"//"))) {
		int len=strlen(mf->filename)-(tmp-mf->filename);
		if (len>0) memmove(tmp,tmp+1,len);
	}
	if (mf->filename) new->filename=strdup(mf->filename);

	new->Artist=mf->Artist;
	new->Album=mf->Album;
	new->size=mf->size;
	new->tracklen=mf->tracklen;
	new->track_nr=mf->track_nr;
	new->tracks=mf->tracks;
	new->bitrate=mf->bitrate;
	new->year=mf->year;
	return(new);
}

/*
 * Database counting functions
 */

int count_songs_by_artist (int Artist) {
	MP3File **last;
	int len=0,found=0;
	last=song_db;

	if (Artist < 0) return(0);

	while (last && *last) {
		if (song_db[len]->Artist==Artist) found++;
		last++;len++;
	}
	return(found);
}


int count_songs_by_album (int Artist, int Album) {
	MP3File **last;
	int len=0,found=0;
	last=song_db;

	if (Artist < 0 || Album < 0 ) return(0);

	while (last && *last) {
		if (song_db[len]->Artist==Artist && 
				song_db[len]->Album==Album ) 
			found++;
		last++;len++;
	}
	return(found);
}

/*
 * Database access / update 
 */

void update_songs_via_album (int Artist, int Album) {
	MP3File **last;
	int len=0;
	last=song_db;

	if (Artist < 0) return;
	if (Album < 0) return;

	while (last && *last) {
		if (	song_db[len]->Artist==Artist  &&
			song_db[len]->Album==Album) {
			song_db[len]->tracks= album_db[Album]->tracks;
		}
		last++;len++;
	}
}

/*
 * Database access / add
 */

int get_artist_id (char *artist) {
	Artist **last,*new;
	int len=0;
	last=artist_db;
	
	while (last && *last) {
		if (strcmp (artist_db[len]->artist,artist) == 0) return(len);
		last++;len++;
	}
	new=(Artist *) calloc(1,sizeof(Artist));
	new->artist=strdup(artist);
		
	artist_db=realloc(artist_db, (len+2)*sizeof(Artist*));
	artist_db[len]=new;
	artist_db[(len+1)]=NULL;
	max_artists=len+1;
	return(len);

}

int get_album_id (int Artist, char *album) {
	Album **last,*new;
	int len=0;
	last=album_db;

	while (last && *last) {
		if (strcmp (album_db[len]->album,album) == 0 && 
				album_db[len]->Artist==Artist) return(len);
		last++;len++;
	}
	new=(Album *) calloc(1,sizeof(Album));
	new->album=strdup(album);
	new->Artist=Artist;
		
	album_db=realloc(album_db, (len+2)*sizeof(Album*));
	album_db[len]=new;
	album_db[(len+1)]=NULL;
	max_albums=len+1;
	return(len);

}
//#define USE_HASH
#ifdef USE_HASH

#include <sys/types.h>
#include <limits.h>
#include <db.h>
#include <fcntl.h>

 DB *dbp = NULL;

void hash_init (void) {
	int ret;

	if (dbp) dbp->close (dbp, 0);
	printf("(re) init hash database\n");

	/* Create the database handle and open the underlying database. */
	if ((ret = db_create (&dbp, NULL, 0)) != 0) {
		fprintf (stderr, "db_create: %s\n", db_strerror (ret));
		exit (1);
	}
	if ((ret = dbp->set_flags (dbp, DB_DUP)) != 0) {
		printf("DUP set failed\n");
	}
	if ((ret = dbp->open (dbp, NULL, NULL, DB_BTREE, DB_CREATE, 0664)) != 0) {
		dbp->err (dbp, ret, "db_open");
		dbp->close (dbp, 0);
		dbp=NULL;
	}
}

void hashadd (MP3File *mf, int idx) {
	DBT key, data;
	int *myidx;
	if (!dbp) return;


	myidx=malloc(sizeof(int));
	*myidx=idx;

	/* Initialize key/data structures. */
	memset (&key, 0, sizeof (key));
	memset (&data, 0, sizeof (data));

	key.data = strdup(mf->songname);
	key.size = 1+strlen (key.data);
	data.data = myidx;
	data.size = sizeof (int);

	if(dbp->put (dbp, NULL, &key, &data, 0)!=0)
		printf("db error\n");

//	printf("added %s %i %p\n",(char*)key.data,*((int*)data.data),data.data);
	free((char*)key.data);
	free((int*)data.data);
}

int hashsearch (MP3File *mf) {
	DBC *dbc;
	DBT key, data;
	      
	if (!dbp) hash_init();
	if (!dbp) return(-1);

	/* Initialize key/data structures. */
	memset (&key, 0, sizeof (key));
	memset (&data, 0, sizeof (data));

	key.data = strdup(mf->songname);
	key.size = 1+strlen (key.data);

	dbp->cursor(dbp,NULL,&dbc,0);
	if (dbc->c_get (dbc,  &key, &data, DB_SET) == 0) {
		do {
			int idx=*((int*)data.data);
			if (idx<max_songs && 
				strcmp (song_db[idx]->songname,mf->songname) == 0 && 
				song_db[idx]->Artist==mf->Artist  &&
				song_db[idx]->Album==mf->Album) {
					free((char*)key.data);
					return(idx);
			}

		}while (dbc->c_get (dbc, &key, &data, DB_DUP) == 0);
	}
	free((char*)key.data);
	return(-1);
}

#endif

int get_song_id (MP3File *mf) {
	MP3File **last;
	int len=0;
	last=song_db;

	if (mf->Artist < 0) return(-1);

#ifdef USE_HASH
	len=hashsearch(mf);
	if (len<0) len=max_songs;
	else return(len);
#else
	while (last && *last) {
		if (strcmp (song_db[len]->songname,mf->songname) == 0 && 
				song_db[len]->Artist==mf->Artist  &&
				song_db[len]->Album==mf->Album) return(len);
		last++;len++;
	}
#endif

	song_db=realloc(song_db, (len+2)*sizeof(MP3File*));
	song_db[len]=dup_mf(mf);
	song_db[(len+1)]=NULL;
	max_songs=len+1;
#ifdef USE_HASH
	hashadd(mf,len);
#endif
	return(len);

}

/*
 * File I/O and parsing 
 */

void clear_mf (MP3File *mf) {
	mf->Artist=-1;
	mf->Album=-1;
	mf->Album=-1;
}

void update_index(int Artist, int Album, char *root, char *filename) {
	FILE *fp;
	char *m3ufile;
	char *path;
	int line,tracks;
	char buf[PATH_MAX],*ext;

	asprintf(&m3ufile,"%s/%s",root, filename);
	asprintf(&path,"%s/%s",root, filename);
	if ((ext=strrchr (path, '/'))) *ext=0;

	/* attempt to open playlist file */
	if (!(fp = fopen (m3ufile, "r"))) {
		free(m3ufile);
		return;
	}
	/* for now assume that all playlist files will be line-based
	   all of these are line based -- add different code for different
	   playlist files */

	line = -1; /* nr of line being read */
	tracks = 0; /* nr of tracks added */

	while (fgets (buf, PATH_MAX, fp)) {

		char *bufp = buf;
		char *tmp;
		int song;
		int len = strlen (bufp); /* remove newline */
		MP3File *mf;

		++line;
		if (len == 0) continue;   /* skip empty lines */
		if (bufp[len-1] == 0x0a)  bufp[len-1] = 0;
		if (*bufp == '#') continue;
		/* assume the rest of the line is a filename */

		if (!(ext=strrchr (bufp, '.'))) continue;

		++tracks;
		mf=(MP3File *) calloc(1,sizeof(MP3File));
		mf->Artist=Artist; mf->Album=Album;
		asprintf(&(mf->filename),"%s/%s",path, bufp);
		*ext=0; // strip extension
		mf->songname=strdup(bufp);
		while ((tmp=strchr(mf->songname,'_'))) *tmp=' '; 

		song=get_song_id(mf);
		(song_db[song])->track_nr=tracks;
		free_mf(mf);
	}

	album_db[Album]->tracks=tracks;
	album_db[Album]->playlist=strdup(m3ufile);

	update_songs_via_album(Artist,Album);

	fclose(fp);
	free(m3ufile);
	free(path);
}

void parse_filename (MP3File *mf, char *root, char *filename)
{
	char *ext;
	char *start,*end;
	char *info[8];
	char *tmp;
	int i=0, cnt;
	start=filename;

	while (*start=='/') start++;
	while (start && i<7 && (end=strchr(start,'/'))) {
		*end='\0';
		asprintf(&(info[i]),"%s",start);
		*end='/'; start=end+1;
		// replace '_' with ' '  
		while ((tmp=strchr(info[i],'_'))) *tmp=' '; 
		i++;
	}
	asprintf(&(info[i]),"%s",start);
	while ((tmp=strchr(info[i],'_'))) *tmp=' '; 

	cnt=i; i=0;

	clear_mf(mf);
	/* parse extension */

	if (!(ext=strrchr (info[cnt], '.'))) return;
	if (strcmp(ext,".mp3")==0 || 
			strcmp(ext,".m4a")==0 || 
			strcmp(ext,".mp4")==0  ) {
		//printf("mp3: %i ",cnt);
		;
	}
	else if (strcmp(ext,".m3u")==0) {
		/* update album index */
		int Artist,Album;
		if (cnt!=2 || strcmp(info[2],prefs->indexname)) goto out;
		Artist= get_artist_id(info[0]); 
		Album= get_album_id(Artist, info[1]); 
		update_index(Artist,Album,root,filename);
		goto out;
	}
	else if (strcmp(ext,".jpg")==0) {
		/* update album cover art */
		int Artist,Album;
		if (cnt!=2 || strcmp(info[2],prefs->imagename)) goto out;
		Artist= get_artist_id(info[0]); 
		Album= get_album_id(Artist, info[1]); 
		asprintf(&(album_db[Album]->coverimage),"%s/%s",root, filename);
		goto out;
	}
	else {
		printf("UNKNOWN FILE:  ...%s\n",filename);
		goto out;
	}

	if (ext) *ext='\0'; // strip extension

	/* apply Info */
	start=filename;
	while (*start=='/') start++;

	if (cnt == 2 ) {
		int Artist= get_artist_id(info[0]); 
		int Album= get_album_id(Artist, info[1]); 
		mf->songname=strdup(info[2]); 
		asprintf(&(mf->filename),"%s/%s",root, start);
		mf->Artist=Artist;
		mf->Album=Album;
	} 
	else if ( cnt == 1 ) {
		int Artist= get_artist_id(info[0]); 
		mf->songname=strdup(info[1]); 
		asprintf(&(mf->filename),"%s/%s",root, start);
		mf->Artist=Artist;
		mf->Album=-1;
	} else {
		printf("\n!!could not parse INFO : %i ",cnt);
	}

out:
	while (i<=cnt) free(info[i++]);

}

void db_add_file (char *root, char *filename)
{
	MP3File *mf;
	int sid=-2;
	mf=(MP3File *) calloc(1,sizeof(MP3File));
	parse_filename(mf,root,filename);
	if (mf->Artist>=0) sid=get_song_id(mf);
	free_mf(mf);
}

//#define MP3ROOT "/home/data/mp3/mp3-a"
//#define MP3ROOT "/home/data/fake"
#define MP3ROOT (prefs->root)

void parse_dir (char *root, char *path)
{
	DIR *dir;
	struct dirent *dirent;
	struct stat statbuf;
	char *npath, *fpath;

	asprintf(&fpath,"%s/%s",root, path);

	if ((dir = opendir(fpath)) != NULL)
	{
		while ((dirent = readdir(dir)) != NULL)
		{
			if (dirent->d_name[0] == '.')
				continue;

			asprintf(&npath,"%s/%s",fpath, dirent->d_name);
			if (stat(npath, &statbuf)== -1) {
				g_free(npath);
				continue;
			}

			free(npath); asprintf(&npath,"%s/%s",path, dirent->d_name);
/*
			if ((++totalfiles)%17==0) {
				status_printf(
					"scanned %i Files. Database: %i Songs [Artists:%i Albums:%i]",
					totalfiles,max_songs,max_artists,max_albums);
					while (gtk_events_pending ()) gtk_main_iteration();
			}
*/
			++totalfiles;
			while (gtk_events_pending ()) gtk_main_iteration();

			if ( S_ISREG(statbuf.st_mode) || S_ISLNK(statbuf.st_mode) )
				db_add_file(root,npath);
			if  ( S_ISDIR(statbuf.st_mode)) 
				parse_dir(root,npath);
			free(npath);
		}
		closedir(dir);
	}
	free(fpath);
}


//static gpointer build_progress_thread (gpointer gp)
gboolean build_progress_thread (gpointer gp)
{
	if (!build_active) return (0);
	status_printf( "scanned %i Files. Database: %i Songs [Artists:%i Albums:%i]",
		totalfiles,max_songs,max_artists,max_albums);
	return(1);
}


void build_db (void)
{
	if (build_active) {
		status_msg("database build already in progress!");
		return;
	}
	clear_db();
	//FIXME: also clear lists.. ??
	status_msg("building database...");

	build_active = 1;
//	g_timeout_add(1000,build_progress_thread,NULL);
	g_timeout_add(200,build_progress_thread,NULL);
	 
	totalfiles=0;
	//FIXME: add support for multiple data sources.
//	printf("debug parsing: %s \n",MP3ROOT);
	parse_dir(MP3ROOT,"/");


	build_active = 0;
	update_list_artist();
	status_default();
}

/*
 * ext db 
 *
 * so far all extra information is stored somewhere 
 * independent of the database !!
 * 
 * so we need to parse and sync this one to the db.
 * this is *not* the final sollution... 
 * evtl. iTunesDB or gpm / sqllite , xmms2,  STAY TUNED.
 */

int get_song_by_filename (char *filename) {
	MP3File **last;
	int len=0, ret=-1;
	char *tmp;
	char *fn=strdup(filename);
	// process fn : replace  '//' by '/', strip newline,...
	if ((tmp=strchr(fn,'\n'))) *tmp=0;
	if ((tmp=strchr(fn,'\r'))) *tmp=0;

	while ((tmp=strstr(fn,"//"))) {
		int len=strlen(fn)-(tmp-fn);
		if (len>0) memmove(tmp,tmp+1,len);
	}
//	printf("SNC: %s - ",fn);

	last=song_db;
	while (last && *last) {
		if (strstr(fn,(*last)->filename)){
			ret=len;
		}
		last++;len++;
	}
	free(fn);
	return(ret);
}
