#define _GNU_SOURCE


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "preferences.h"

Preferences *prefs;

void init_preferences(void) {
	prefs=calloc(1,sizeof(Preferences));
	prefs->root=strdup("/home/data/mp3/mp3-a");
	prefs->searchlimit=500;
	prefs->restore=TRUE;
	prefs->auto_select_album=2;
	prefs->auto_save_db=0;
	prefs->xmms=FALSE;
	prefs->xmms_start=FALSE;
	prefs->add_random=1;
	prefs->add_random_two=FALSE;
	prefs->playlist_maxlen=0;
	prefs->indexname=strdup("00index.m3u");
	prefs->imagename=strdup("00record.jpg");
}

void set_preferences(GtkWidget *prefdialog) {
	GtkWidget *widget;
	widget=lookup_widget(prefdialog,"entry_root");
	gtk_entry_set_text (GTK_ENTRY (widget), prefs->root);
	widget=lookup_widget(prefdialog,"spin_searchlimit");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget), (gdouble) prefs->searchlimit);
	widget=lookup_widget(prefdialog,"check_restore");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), prefs->restore);
	if (prefs->auto_select_album==0) 
		widget=lookup_widget(prefdialog,"radio_none");
	else if (prefs->auto_select_album==1) 
		widget=lookup_widget(prefdialog,"radio_all");
	else widget=lookup_widget(prefdialog,"radio_album");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), TRUE);
	widget=lookup_widget(prefdialog,"check_savedb");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), prefs->auto_save_db);
	widget=lookup_widget(prefdialog,"check_random");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), prefs->add_random);
	widget=lookup_widget(prefdialog,"check_plsend");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), prefs->add_random_two);
	widget=lookup_widget(prefdialog,"check_xmms");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), prefs->xmms);
	widget=lookup_widget(prefdialog,"check_xmms_start");
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), prefs->xmms_start);
	widget=lookup_widget(prefdialog,"spin_playlistmax");
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(widget), (gdouble) prefs->playlist_maxlen);
	widget=lookup_widget(prefdialog,"entry_index");
	gtk_entry_set_text (GTK_ENTRY (widget), prefs->indexname);
	widget=lookup_widget(prefdialog,"entry_image");
	gtk_entry_set_text (GTK_ENTRY (widget), prefs->imagename);
}

void parse_preferences(GtkWidget *prefdialog) {
	GtkWidget *widget;

	widget=lookup_widget(prefdialog,"entry_root");

	if (prefs->root) free(prefs->root);
	prefs->root=strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
	widget=lookup_widget(prefdialog,"spin_searchlimit");
	prefs->searchlimit = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	widget=lookup_widget(prefdialog,"check_restore");
	prefs->restore=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(prefdialog,"radio_all");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget)))
		prefs->auto_select_album=1;
	widget=lookup_widget(prefdialog,"radio_none");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget)))
		prefs->auto_select_album=0;
	widget=lookup_widget(prefdialog,"radio_album");
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget)))
		prefs->auto_select_album=2;
	widget=lookup_widget(prefdialog,"check_savedb");
	prefs->auto_save_db=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(prefdialog,"check_random");
	prefs->add_random=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(prefdialog,"check_plsend");
	prefs->add_random_two=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(prefdialog,"check_xmms");
	prefs->xmms=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(prefdialog,"check_xmms_start");
	prefs->xmms_start=gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(prefdialog,"spin_playlistmax");
	prefs->playlist_maxlen = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	widget=lookup_widget(prefdialog,"entry_index");
	if (prefs->indexname) free(prefs->indexname);
	prefs->indexname=strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
	widget=lookup_widget(prefdialog,"entry_image");
	if (prefs->imagename) free(prefs->imagename);
	prefs->imagename=strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
}


/* Returns "$HOME/.NAME" or NULL;
 * You must g_free the string after use */
gchar *get_home_cfg (char *name)
{
	G_CONST_RETURN gchar *str;
	gchar *cfgdir=NULL;

	if((str = g_get_home_dir ())) 
		cfgdir = g_build_filename (str, name, NULL);
	return cfgdir;
}


void save_preferences() {
	gchar *fn;
	FILE *F;
	fn=get_home_cfg(".mp3browser.cfg");

	if(!(F=fopen(fn,"w+"))) {
		fprintf(stderr,"could not open file for writing : '%s'\n",fn);
		return;
	}
	fprintf(F,"[mp3b]\n");
	fprintf(F,"root=%s\n",prefs->root);
	fprintf(F,"searchlimit=%i\n",prefs->searchlimit);
	fprintf(F,"restore=%i\n",prefs->restore?1:0);
	fprintf(F,"auto_select_album=%i\n",prefs->auto_select_album);
	fprintf(F,"auto_save_db=%i\n",prefs->auto_save_db?1:0);
	fprintf(F,"add_random=%i\n",prefs->add_random?1:0);
	fprintf(F,"xmms=%i\n",prefs->xmms?1:0);
	fprintf(F,"xmms_start=%i\n",prefs->xmms_start?1:0);
	fprintf(F,"add_random_two=%i\n",prefs->add_random_two?1:0);
	fprintf(F,"playlist_maxlen=%i\n",prefs->playlist_maxlen);
	fprintf(F,"indexname=%s\n",prefs->indexname);
	fprintf(F,"imagename=%s\n",prefs->imagename);
	fclose(F);
	g_free(fn);	
}

void load_preferences() {
	gchar *fn;
	FILE *F;
	char buf[PATH_MAX];

	fn=get_home_cfg(".mp3browser.cfg");
	if(!(F=fopen(fn,"r"))) {
		fprintf(stderr,"could not open file for reading : '%s'\n",fn);
		return;
	}

	while (fgets(buf,PATH_MAX,F)) {
		char *tmp;
		if ((tmp=strchr(buf,'\n'))) *tmp=0;
		if (buf[0]=='[') continue;
		if (strncmp(buf,"root=",5)==0) {
			if (prefs->root) free(prefs->root);
			prefs->root=strdup(buf+5);
		}
		if (strncmp(buf,"searchlimit=",12)==0) 
			prefs->searchlimit=atoi(buf+12);
		if (strncmp(buf,"restore=",8)==0) 
			prefs->restore=atoi(buf+8)?TRUE:FALSE;
		if (strncmp(buf,"auto_select_album=",18)==0) 
			prefs->auto_select_album=atoi(buf+18);
		if (strncmp(buf,"auto_save_db=",13)==0) 
			prefs->auto_save_db=atoi(buf+13)?TRUE:FALSE;
		if (strncmp(buf,"add_random=",11)==0) 
			prefs->add_random=atoi(buf+11)?TRUE:FALSE;
		if (strncmp(buf,"xmms=",5)==0) 
			prefs->xmms=atoi(buf+5)?TRUE:FALSE;
		if (strncmp(buf,"xmms_start=",11)==0) 
			prefs->xmms_start=atoi(buf+11)?TRUE:FALSE;
		if (strncmp(buf,"add_random_two=",15)==0) 
			prefs->add_random_two=atoi(buf+15)?TRUE:FALSE;
		if (strncmp(buf,"playlist_maxlen=",16)==0) 
			prefs->playlist_maxlen=atoi(buf+16);
		if (strncmp(buf,"indexname=",10)==0) {
			if (prefs->indexname) free(prefs->indexname);
			prefs->indexname=strdup(buf+10);
		}
		if (strncmp(buf,"imagename=",10)==0) {
			if (prefs->imagename) free(prefs->imagename);
			prefs->imagename=strdup(buf+10);
		}
	}
	fclose(F);
	g_free(fn);	

}
