#define _GNU_SOURCE

#define INCLUDE_DND

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "mp3db.h"
#include "search.h"
#include "display.h"
#include "preferences.h"

extern GtkWidget *mainwindow;
extern GtkWidget *searchdialog;

extern Preferences *prefs;

extern int p_artist;
extern int p_album;

extern int build_active;
gboolean search_select= FALSE;

enum { SRC_ARTIST = 0, SRC_ALBUM, SRC_SONG, SRC_IDX_A, SRC_IDX_R, SRC_IDX_S, SRC_LAST };
GtkListStore *ls_search=NULL;

extern Artist **artist_db;
extern Album **album_db;
extern MP3File **song_db;

/*
 * Drag and Drop (Song)
 */

static void
on_ts_dnd_get_uri_foreach(GtkTreeModel *tm, GtkTreePath *tp,
			  GtkTreeIter *iter, gpointer data)
{
	int idx;
	GString *filelist = (GString *)data;
	char *name=NULL;

	gtk_tree_model_get(tm, iter, SRC_IDX_S, &idx, -1);
	if (idx>=0) name = song_db[idx]->filename;
	if (name)
	{
		gchar *uri = g_filename_to_uri (name, NULL, NULL);
		if (uri)
		{
			g_string_append_printf (filelist, "%s\n", uri);
			g_free (uri);
		}
	}
}

static void
on_ts_dnd_get_file_foreach(GtkTreeModel *tm, GtkTreePath *tp,
			   GtkTreeIter *iter, gpointer data)
{
	int idx;
	GString *filelist = (GString *)data;
	char *name=NULL;

	gtk_tree_model_get(tm, iter, SRC_IDX_S, &idx, -1);
	if (idx>=0) name = song_db[idx]->filename;
	if (name) g_string_append_printf (filelist, "file:%s\n", name);
}


static void ts_drag_data_get (GtkWidget       *widget,
			      GdkDragContext  *context,
			      GtkSelectionData *data,
			      guint            info,
			      guint            time,
			      gpointer         user_data)
{
    GtkTreeSelection *ts = NULL;
    GString *reply = g_string_sized_new (2047);


    if((data) && (ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(widget))))
    {
	switch (info)
	{
	case DND_TEXT_URI_LIST:
	    gtk_tree_selection_selected_foreach(ts,
				    on_ts_dnd_get_uri_foreach, reply);
	    break;
	case DND_TEXT_PLAIN:
	    gtk_tree_selection_selected_foreach(ts,
				    on_ts_dnd_get_file_foreach, reply);
	    break;
	default:
	    g_warning ("Programming error: ts_drag_data_get received unknown info type (%d)\n", info);
	    break;
	}
    }
    gtk_selection_data_set(data, data->target, 8, (guchar*) reply->str, reply->len);
    g_string_free (reply, TRUE);
}

		
/*
 * 'selection changed' callback
 */

gboolean func_search_sel (GtkTreeSelection *selection,
				GtkTreeModel     *model,
				GtkTreePath      *path,
				gboolean          path_currently_selected,
				gpointer          userdata)
{
	GtkTreeIter iter;
	if (gtk_tree_model_get_iter(model, &iter, path))
	{
		int artist, album, song;
		gtk_tree_model_get(model, &iter, SRC_IDX_A, &artist, -1);
		gtk_tree_model_get(model, &iter, SRC_IDX_R, &album, -1);
		gtk_tree_model_get(model, &iter, SRC_IDX_S, &song, -1);

		if (!path_currently_selected) 
		{
			search_select=TRUE;
			select_item("tv_artist",ART_IDX,artist);
			select_item("tv_album",ALB_IDX,album);
			select_item("tv_song",COL_IDX,song);
			search_select=FALSE;
		}
	}
	return TRUE; /* allow selection state to change */
}

/*
 * Search function
 */

#define SEARCHLIMIT (((prefs->searchlimit)<1) || (found < (prefs->searchlimit)))

void do_search(void) {
	GtkWidget *widget;
	GtkTreeView *tv_search;
	gboolean c_album,c_artist,c_song;
	char *search;
	GtkTreeIter iter;
	int found=0;
	int total=0;

	if (build_active) return;

	widget=lookup_widget(searchdialog,"checkbutton_album");
	c_album=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(searchdialog,"checkbutton_artist");
	c_artist=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(searchdialog,"checkbutton_song");
	c_song=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	widget=lookup_widget(searchdialog,"entry_search");
	search=strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
/*	
	printf("search: '%s' in [%s%s%s]\n",search,
			c_artist?"Artists ":"",
			c_album?"Albums ":"",
			c_song?"Songs ":"");
*/

	// do search
	
	gtk_list_store_clear(ls_search);
	if (strlen(search)==0) goto out;

	status_msg("searching...");
	while (gtk_events_pending ()) gtk_main_iteration();

	tv_search=GTK_TREE_VIEW(lookup_widget(searchdialog,"tv_search"));
	g_object_ref(ls_search); /* Make sure the model stays with us after the tree view unrefs it */
	gtk_tree_view_set_model (tv_search, NULL ); /* speed up insertion process */

	if(c_artist)
	{
		Artist **last;
		int idx=0;
		last=artist_db;
		while (last && *last) {
			if (strcasestr((*last)->artist,search)) {
				total++; if (!SEARCHLIMIT) { last++;idx++; continue;}
				gtk_list_store_append (ls_search, &iter);
				gtk_list_store_set (ls_search, &iter,
						SRC_ARTIST,(*last)->artist,
						SRC_ALBUM,"",
						SRC_SONG,"",
						SRC_IDX_A,idx,
						SRC_IDX_R,-1,
						SRC_IDX_S,-1,
						-1);
				found++;
			}
			last++;idx++;
		}
	}
	if(c_album)
	{
		Album **last;
		int idx=0;
		last=album_db;
		while (last && *last) {
			if (strcasestr((*last)->album,search)) {
				total++; if (!SEARCHLIMIT) { last++;idx++; continue;}
				gtk_list_store_append (ls_search, &iter);
				gtk_list_store_set (ls_search, &iter,
						SRC_ARTIST,artist_db[(*last)->Artist]->artist,
						SRC_ALBUM,(*last)->album,
						SRC_SONG,"",
						SRC_IDX_A,(*last)->Artist,
						SRC_IDX_R,idx,
						SRC_IDX_S,-1,
						-1);
				found++;
			}
			last++;idx++;
		}
	}
	if(c_song)
	{
		MP3File **last;
		int idx=0;
		last=song_db;
		while (last && *last) {
			if (strcasestr((*last)->songname,search)) {
				int album=(*last)->Album;
				total++; if (!SEARCHLIMIT) { last++;idx++; continue;}
				gtk_list_store_append (ls_search, &iter);
				gtk_list_store_set (ls_search, &iter,
						SRC_ARTIST,artist_db[(*last)->Artist]->artist,
						SRC_ALBUM,album<0?"(no album)":album_db[(*last)->Album]->album,
						SRC_SONG,(*last)->songname,
						SRC_IDX_A,(*last)->Artist,
						SRC_IDX_R,(*last)->Album,
						SRC_IDX_S,idx,
						-1);
				found++;
			}
			last++;idx++;
		}
	}

	gtk_tree_view_set_model (tv_search, GTK_TREE_MODEL (ls_search));
	g_object_unref(ls_search); 
out:
	if (!SEARCHLIMIT) 
		status_printf("STOPPED at %i of %i results (see preferences)",found,total);
	else 
		status_printf("%i results searching for '%s' in [ %s%s%s]",found,search,
			c_artist?"Artists ":"",
			c_album?"Albums ":"",
			c_song?"Songs ":"");
	free(search);
}


/* 
 * Widget init
 */

void init_tv_search (void) {
	GtkTreeView *tv_search;
	GtkTreeViewColumn   *col;
	GtkCellRenderer     *renderer;
	GtkTreeSortable *sortable;
	GtkTreeSelection *selection;

	ls_search = gtk_list_store_new (SRC_LAST, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);
	sortable = GTK_TREE_SORTABLE(ls_search);
	
	gtk_tree_sortable_set_sort_func(sortable, SRC_ARTIST, sort_compare_func, 
			GINT_TO_POINTER(SORTID_ARTIST), NULL);
	gtk_tree_sortable_set_sort_func(sortable, SRC_ALBUM , sort_compare_func,
			GINT_TO_POINTER(SORTID_ALBUM), NULL);
	gtk_tree_sortable_set_sort_func(sortable, SRC_SONG , sort_compare_func,
			GINT_TO_POINTER(SORTID_SONG), NULL);
	gtk_tree_sortable_set_sort_column_id(sortable, SRC_ARTIST, GTK_SORT_ASCENDING);
	

	tv_search=GTK_TREE_VIEW(lookup_widget(searchdialog,"tv_search"));
	gtk_tree_view_set_model (tv_search, GTK_TREE_MODEL (ls_search));
	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (tv_search), GTK_SELECTION_MULTIPLE);

	gtk_drag_source_set(GTK_WIDGET(tv_search),GDK_BUTTON1_MASK,
			drag_types, TGNR(drag_types),GDK_ACTION_COPY);
	gtk_signal_connect (GTK_OBJECT(tv_search), "drag_data_get", G_CALLBACK(ts_drag_data_get),NULL);


	selection = gtk_tree_view_get_selection(tv_search);
        gtk_tree_selection_set_select_function(selection, func_search_sel, NULL, NULL);

	/* column 1 */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Artist");
	gtk_tree_view_append_column(tv_search, col);
	gtk_tree_view_column_set_resizable(col,TRUE);

        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", SRC_ARTIST);
	gtk_tree_view_column_set_sort_column_id(col,SRC_ARTIST);

	/* column 2 */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Album");
	gtk_tree_view_append_column(tv_search, col);
	gtk_tree_view_column_set_resizable(col,TRUE);

        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", SRC_ALBUM);
	gtk_tree_view_column_set_sort_column_id(col,SRC_ALBUM);

	/* column 3 */
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(col, "Song");
	gtk_tree_view_append_column(tv_search, col);
	gtk_tree_view_column_set_resizable(col,TRUE);

        renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", SRC_SONG);
	gtk_tree_view_column_set_sort_column_id(col,SRC_SONG);
}
