#define _GNU_SOURCE

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "mp3db.h"
#include "search.h"
#include "display.h"
#include "preferences.h"

extern GtkWidget *mainwindow;
extern GtkWidget *searchdialog;

extern Preferences *prefs;

extern int p_artist;
extern int p_album;

extern int build_active;

extern Artist **artist_db;
extern Album **album_db;
extern MP3File **song_db;

extern int max_songs,max_artists,max_albums;

int save_db (char *fn) {
	FILE *F;

	if(!(F=fopen(fn,"w+"))) {
		fprintf(stderr,"could not open file for writing : '%s'\n",fn);
		return (-1);
	}
	fprintf(F,"[MP3database]\n");
	{
		Artist **last;
		int idx=0;
		last=artist_db;
		fprintf(F,"[Artist]\n");
		while (last && *last) { 
			fprintf(F,"%i;%s\n",idx,(*last)->artist);
			idx++;
			last++;
		}
	}
	{
		Album **last;
		int idx=0;
		last=album_db;
		fprintf(F,"[Album]\n");
		while (last && *last) { 
			fprintf(F,"%i;%i;%s;%s;%s\n",idx,
					(*last)->Artist,
					(*last)->album,
					(*last)->coverimage,
					(*last)->playlist);
			idx++;
			last++;
		}
	}
	{
		MP3File **last;
		int idx=0;
		last=song_db;
		fprintf(F,"[Song]\n");
		while (last && *last) { 
			fprintf(F,"%i;%i;%i;%s;%s;%i;%i\n",idx,
					(*last)->Artist,
					(*last)->Album,
					(*last)->songname,
					(*last)->filename,
					(*last)->track_nr,
					(*last)->tracks);
			idx++;
			last++;
		}
	}
	fclose(F);
	return (0);
}

enum {MODE_ARTIST, MODE_ALBUM, MODE_SONG, MODE_HEADER, MODE_UNKNOWN };
#define PB(NUM)  {printf("DB read failed (error: %i)\n",NUM); error=1; break;}
#define NONULL(STRING) {if(strcmp(STRING,"(null)")==0) {free(STRING); STRING=NULL;}}

int load_db (char *fn) {

	FILE *F;
	char buf[PATH_MAX];
	int mode,error;

	Artist *artist;
	Album *album;
	MP3File *mf;

	if (build_active) {
		status_msg("database build already in progress!");
		return(-1);
	}

	if(!(F=fopen(fn,"r"))) {
		fprintf(stderr,"could not open file for reading : '%s'\n",fn);
		return (-1);
	}
	build_active = 1;
	clear_db();

	mode=MODE_UNKNOWN;
	error=0;

	while (!error && fgets(buf,PATH_MAX,F)) {
		char *tmp;
		if ((tmp=strchr(buf,'\n'))) *tmp=0;

		if (buf[0]=='[') {
			if (strncmp(buf,"[Artist]",8)==0) mode= MODE_ARTIST; 
			else if (strncmp(buf,"[Album]",7)==0) mode= MODE_ALBUM;
			else if (strncmp(buf,"[Song]",6)==0) mode= MODE_SONG;
			else if (strncmp(buf,"[MP3database]",13)==0) mode= MODE_HEADER;
			else mode=MODE_UNKNOWN;
		} else {
			int idx_a,idx_r,idx_s;
			int len;
			char *start,*end;
			start=end=NULL;

			switch (mode) 
			{
			case MODE_ARTIST:
				len=max_artists;
				artist=(Artist *) calloc(1,sizeof(Artist));
				idx_a=atoi(buf);
				if (!(start=strchr(buf,';'))) PB(400)
				asprintf(&(artist->artist),"%s",start+1);
				
				if (idx_a != len) PB(401)

				artist_db=realloc(artist_db, (len+2)*sizeof(Artist*));
				artist_db[len]=artist;
				artist_db[++len]=NULL;
				max_artists=len;
				break;
			case MODE_ALBUM:
				len=max_albums;
				album=(Album *) calloc(1,sizeof(Album));
				idx_r=atoi(buf);

				if (!(start=strchr(buf,';'))) PB(500)
				if(!(end=strchr(start+1,';'))) PB(501)
				*end=0; album->Artist=atoi(start+1); start=end+1;

				if(!start || !(end=strchr(start,';'))) PB(502)
				*end=0; asprintf(&(album->album),"%s",start); start=end+1;
				if(!start || !(end=strchr(start,';'))) PB(503)
				*end=0; asprintf(&(album->coverimage),"%s",start); start=end+1;
				if(!start) PB(504)
				asprintf(&(album->playlist),"%s",start); start=end+1;

				if (idx_r != len) PB(505)

				NONULL(album->coverimage);
				NONULL(album->playlist);

				album_db=realloc(album_db, (len+2)*sizeof(Album*));
				album_db[len]=album;
				album_db[++len]=NULL;
				max_albums=len;
				break;
			case MODE_SONG:
				len=max_songs;
				mf=(MP3File *) calloc(1,sizeof(MP3File));
				idx_s=atoi(buf);

				if (!(start=strchr(buf,';'))) PB(600);
				if(!(end=strchr(start+1,';'))) PB(601);
				*end=0; mf->Artist=atoi(start+1); start=end+1;

				if(!start || !(end=strchr(start,';'))) PB(602);
				*end=0; mf->Album=atoi(start); start=end+1;
				if(!start || !(end=strchr(start,';'))) PB(603);
				*end=0; asprintf(&(mf->songname),"%s",start); start=end+1;
				if(!start || !(end=strchr(start,';'))) PB(604);
				*end=0; asprintf(&(mf->filename),"%s",start); start=end+1;
				if(!start || !(end=strchr(start,';'))) PB(605);
				*end=0; mf->track_nr=atoi(start); start=end+1;
				mf->tracks=atoi(start); start=end+1;

				if (idx_s != len) PB(606)

				song_db=realloc(song_db, (len+2)*sizeof(MP3File*));
				song_db[len]=mf;
				song_db[++len]=NULL;
				max_songs=len;
				break;
			default:
				error=1;
				printf("unparsed: %s",buf);
				break;
			}
		}
	}

	build_active = 0;
 return (0);
}
