/* 
   mp3b - xmms2

   Copyright (C) 2007 Robin Gareus <robin@gareus.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 
   as published by the Free Software Foundation;

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

*/

#define _GNU_SOURCE

#include <stdio.h>
#include "preferences.h"
#include "xmms2.h"
#include "mp3db.h"
#include "display.h"

extern int build_active, progress_update;
extern int max_songs,max_artists,max_albums;
extern gboolean search_select;
extern MP3File **song_db;
extern Preferences *prefs;

#ifdef HAVE_XMMS2

xmmsc_connection_t  *connection = NULL;

gchar *xmformat="${artist} - ${title}";

static gboolean has_songname = FALSE;
static guint current_id = 0;
//static gint curr_dur = 0; // unused
//static guint last_dur = 0; // unused
//static gchar songname[256]; // unused
static gchar curr_url[1024];
static guint curr_status = 0;

typedef struct {
  guint playlist_pos;
  guint playlist_length;
  guint total_playtime;
  guint process_tottime;
} plsnfo;

static plsnfo my_playlist = { 0, 0, 0, 0 };

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
/** helpers */

gint res_has_key (xmmsc_result_t *res, const gchar *key) {
  return xmmsc_result_get_dict_entry_type (res, key) != XMMSC_RESULT_VALUE_TYPE_NONE;
}

gchar *format_url (gchar *item, GFileTest test) {
  gchar *url, rpath[PATH_MAX], *p;
  p = strchr (item, ':');
  if (!(p && p[1] == '/' && p[2] == '/')) {
    if (!realpath (item, rpath)) { return NULL; }
    if (!g_file_test (rpath, test)) { return NULL; }
    url = g_strdup_printf ("file://%s", rpath);
  } else {
    url = g_strdup_printf ("%s", item);
  }
  return url;
}
#if 0 // xmms2-media lib 
static void handle_ml_lookup (xmmsc_result_t *res, void *userdata) {
  guint id;
  XMMS2CERR(res)
  xmmsc_result_get_uint (res, &id);
  xmmsc_result_unref (res);
  printf("xmms2-ID:%4i mp3b-ID:%4i # item:%s\n",id, get_song_by_filename((char*)userdata), (char*)userdata);
  // TODO: if we have a cover image, set it..
  // system("xmms2 mlib addcover <cover-img> xmms-id [xmms-id]*");
  g_free (userdata);
}

void ml_lookup (char *item) {
  xmmsc_result_t *res;
  gchar *url = format_url (item, G_FILE_TEST_IS_REGULAR);
  if (!url) { fprintf (stderr, "Invalid url\n"); return; }
  res=xmmsc_medialib_get_id(connection, url);
  xmmsc_result_notifier_set(res, handle_ml_lookup, g_strdup(item));
  xmmsc_result_unref (res);
  g_free (url);
}
#endif

#if 0  // xmms2-volume - unused
void handle_volume_port (const void *key, xmmsc_result_value_type_t type, const void *value, void *userdata) {
  gint setvol = ((gint)userdata);
  if (setvol <0 || setvol >100) { // dump
    printf("volume: %s = %u\n", (char*)key, XPOINTER_TO_UINT (value)); // FIXME
  } else { //set
      printf("set volume: %s = %d\n", (char*)key, setvol);
	xmmsc_result_t *res;
	res = xmmsc_playback_volume_set (connection, (char*)key, setvol);
	xmmsc_result_unref(res);
  }
}

static void  handle_volume_ports (xmmsc_result_t *res, void *userdata) {
  XMMS2CERR(res)
  xmmsc_result_dict_foreach (res, handle_volume_port, userdata);
}

void cmd_volume (xmmsc_connection_t *conn, gint setvol) {
  xmmsc_result_t *res;
  res = xmmsc_playback_volume_get (conn);
  xmmsc_result_notifier_set (res, handle_volume_ports, (void*)setvol);
  xmmsc_result_unref (res);
}
#endif

static void  handle_new_collection (xmmsc_result_t *res, void *userdata) {
  xmmsc_connection_t *conn = userdata;
  xmmsc_result_t *res2;
  xmmsc_coll_t *coll;
  const char *order[] = { "id", NULL };

  XMMS2CERR(res)
  if (!xmmsc_result_get_collection (res, &coll)) { XMMS2BROKEN(" - load playlist\n"); }
  res2 = xmmsc_playlist_add_collection (conn, NULL, coll, order);
  XMMS2CERR(res2)
  xmmsc_result_unref (res2);
  printf("Playlist with %d entries added\n", xmmsc_coll_idlist_get_size (coll));
  xmmsc_result_unref (res);
}

static void add_item_to_playlist (xmmsc_connection_t *conn, gchar *playlist, gchar *item);

static void add_playlist (xmmsc_connection_t *conn, gchar *plist) {

  xmmsc_result_t *res;
  gchar *url;

  url = format_url (plist, G_FILE_TEST_IS_REGULAR);

  res = xmmsc_coll_idlist_from_playlist_file (conn, url);
  xmmsc_result_notifier_set (res, handle_new_collection, conn);
  g_free (url);
  //xmmsc_result_unref (res);
}

void check_pls_ending (guint cur_plpos, guint playlist_length, guint playlist_duration) {
  printf("check: %u/%u (%ums)\n", cur_plpos+1, playlist_length, playlist_duration);
  if ((playlist_length == cur_plpos + (prefs->add_random_two?2U:1U)) && prefs->add_random) {
    int rnd=(int) ((float)max_songs*rand()/(RAND_MAX+1.0));
    if (prefs->playlist_maxlen <1 || playlist_duration/60000 < prefs->playlist_maxlen) {
      printf(" auto enqueue '%s'\n",song_db[rnd]->songname);
      add_item_to_playlist(connection, NULL, song_db[rnd]->filename);
    }
  }
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
/** handlers for callback functions */

static void add_item_to_playlist (xmmsc_connection_t *conn, gchar *playlist, gchar *item) {
  xmmsc_result_t *res;
  gchar *url;

  url = format_url (item, G_FILE_TEST_IS_REGULAR);
  if (!url) { fprintf (stderr, "Invalid url\n"); return; }

  res = xmmsc_playlist_add_url (conn, playlist, url);
  XMMS2CERR(res)
  xmmsc_result_unref (res);
  g_free (url);

  printf("xmms2 added %s\n", item);
}

static void  handle_pls_time (xmmsc_result_t *res, void *userdata) {
  gint playtime;
  plsnfo *p = (plsnfo*) userdata;
  if (xmmsc_result_get_dict_entry_int (res, "duration", &playtime)) {
    p->total_playtime += playtime;
  }
  if (++(p->process_tottime) == p->playlist_length) 
    check_pls_ending(p->playlist_pos, p->playlist_length, p->total_playtime);
}

static void  handle_guint (xmmsc_result_t *res, void *userdata) {
  guint ui;
  XMMS2CERR(res)
  xmmsc_result_get_uint (res, &ui);
  xmmsc_result_unref (res);
  if (userdata) *((guint *)userdata)=ui;
}

static void handle_pls_list(xmmsc_result_t *res, void *userdata) {
  plsnfo *p = (plsnfo*) userdata;
  //if (!connection) return;
  p->playlist_length=p->total_playtime=p->process_tottime=0;
  for (; xmmsc_result_list_valid(res); xmmsc_result_list_next(res)) {
    xmmsc_result_t *info_res;
    guint ui;
    if (!xmmsc_result_get_uint (res, &ui)) { XMMS2BROKEN("") }
    p->playlist_length++;
    info_res = xmmsc_medialib_get_info (connection, ui);
    xmmsc_result_notifier_set (info_res, handle_pls_time, p);
    xmmsc_result_unref(info_res);
  }
  xmmsc_result_unref(res);
}

void update_mp3b_hooks (xmmsc_connection_t *conn) {
  gchar *playlist = NULL;
  xmmsc_result_t *res;

  res = xmmsc_playlist_current_pos (conn, playlist);
  xmmsc_result_notifier_set (res, handle_guint, &(my_playlist.playlist_pos));
  xmmsc_result_unref (res);

  res = xmmsc_playlist_list_entries (conn, playlist);
  xmmsc_result_notifier_set (res, handle_pls_list, &my_playlist);
  xmmsc_result_unref(res);

}

static void do_mediainfo (xmmsc_result_t *res, void *userdata) {
  XMMS2CERR(res)
#if 0 // songname - unused
  if (res_has_key (res, "channel") && res_has_key (res, "title")) {
    xmmsc_entry_format (songname, sizeof (songname), "[stream] ${title}", res);
    has_songname = TRUE;
  } else if (res_has_key (res, "channel")) {
    xmmsc_entry_format (songname, sizeof (songname), "${title}", res);
    has_songname = TRUE;
  } else if (!res_has_key (res, "title")) {
    gchar *url, *filename;
    if (xmmsc_result_get_dict_entry_string (res, "url", &url)) {
      filename = g_path_get_basename (url);
      if (filename) {
	g_snprintf (songname, sizeof (songname), "%s", filename);
	g_free (filename);
	has_songname = TRUE;
      }
    }
  } else {
    xmmsc_entry_format (songname, sizeof (songname), xmformat, res);
    has_songname = TRUE;
  }
  xmmsc_result_get_dict_entry_int (res, "duration", &curr_dur);
  //curr_dur += 500; // round!
#endif

  gchar *url;
  if (xmmsc_result_get_dict_entry_string (res, "url", &url)) {
    g_snprintf (curr_url, sizeof (curr_url), "%s", xmmsc_result_decode_url(res,url));
  }

  xmmsc_result_unref (res);
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
/** registered callback handlers */

#if 0 //unused
static void handle_playtime (xmmsc_result_t *res, void *userdata) {
  xmmsc_result_t *newres;
  guint dur;

  XMMS2CERR(res)
  if (!xmmsc_result_get_uint (res, &dur)) { XMMS2BROKEN("") }

  if (((dur / 1000) % 60) != ((last_dur / 1000) % 60)) {
    last_dur = dur;
    //printf("xmms: duration - %i\n",last_dur);
  }
  newres = xmmsc_result_restart (res);
  xmmsc_result_unref (res);
  xmmsc_result_unref (newres);
}
#endif

static void handle_current_id (xmmsc_result_t *res, void *userdata) {
  xmmsc_connection_t *conn = userdata;
  if (!conn || !connection) return; // workaround to disconnected xmms2
  XMMS2CERR(res)
  if (!xmmsc_result_get_uint (res, &current_id)) { XMMS2BROKEN("") }

  if (current_id) {
    res = xmmsc_medialib_get_info (conn, current_id);
    xmmsc_result_notifier_set (res, do_mediainfo, NULL);
    xmmsc_result_unref (res);
    update_mp3b_hooks(conn);
  }
}

static void handle_status_change (xmmsc_result_t *res, void *userdata) {
  guint new_status;
  XMMS2CERR(res)
  if (!xmmsc_result_get_uint (res, &new_status)) { XMMS2BROKEN("") }
  //xmmsc_result_unref (res); // ?
  if (!connection) return; // workaround to disconnected xmms2
  fprintf(stdout, "xmms2: status %i -> %i\n",curr_status, new_status);
  curr_status = new_status;
}

static void handle_xmms_session (void *userdata) {
  xmmsc_unref(connection);
  connection=NULL;
  update_gui(); // async X callback ?!
}

#if 0 // ununsed
static void handle_mediainfo_update (xmmsc_result_t *res, void *userdata) {
  guint id;
  xmmsc_connection_t *conn = userdata;

  XMMS2CERR(res)
  if (!xmmsc_result_get_uint (res, &id)) { XMMS2BROKEN("") }

  if (id == current_id) {
    res = xmmsc_medialib_get_info (conn, current_id);
    xmmsc_result_notifier_set (res, do_mediainfo, NULL);
    xmmsc_result_unref (res);
    update_mp3b_hooks(conn); // here too?
  }
}
#endif

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
/** public funcions  (INTERNAL API)*/
void *ghandle;

int init_xmms_session (void) {
  gchar *path = NULL;
  gint conn_status;
  if(connection) {
    fprintf(stderr,"xmms2 already connected.\n");
    return 1;
  }

  if (!(connection = xmmsc_init("mp3b"))) {
    fprintf (stderr, "Could not init xmmsc_connection!\n");
  }

  if (!(path = getenv("XMMS_PATH"))) {
    if (path == NULL) asprintf(&path, "unix:///tmp/xmms-ipc-%s", g_get_user_name());
  }

  //if (!path || g_ascii_strncasecmp(path, "unix://", 7) == 0) local_xmms2d = TRUE;
  conn_status = xmmsc_connect(connection, path);

  if (!conn_status && prefs->xmms_start) {
    if (!path || g_ascii_strncasecmp(path, "unix://", 7) == 0) {
      fprintf (stdout, "starting up xmms2 daemon.\n");
      if (!system("xmms2-launcher")) {
	conn_status = xmmsc_connect(connection, path);
      }
    }
  }

  if (!conn_status) {
    fprintf(stderr, "Could not connect to xmms2d: %s\n", xmmsc_get_last_error (connection));
    xmmsc_unref (connection);
    connection=NULL;
    return (0);
  } 

  // Setup onchange signal for mediainfo 

  has_songname = FALSE;
  memset(curr_url,0,sizeof(curr_url));
  XMMS_CALLBACK_SET (connection, xmmsc_broadcast_playback_current_id, handle_current_id, connection);
  XMMS_CALLBACK_SET (connection, xmmsc_broadcast_playback_status, handle_status_change, NULL);
  XMMS_CALLBACK_SET (connection, xmmsc_playback_current_id, handle_current_id, connection);
  XMMS_CALLBACK_SET (connection, xmmsc_playback_status, handle_status_change, NULL);
#if 0
  XMMS_CALLBACK_SET (connection, xmmsc_broadcast_medialib_entry_changed, handle_mediainfo_update, connection);
  XMMS_CALLBACK_SET (connection, xmmsc_signal_playback_playtime, handle_playtime, NULL);
#endif
  xmmsc_disconnect_callback_set (connection, handle_xmms_session, NULL); // FIXME / TODO   dis/reconnect
  
  ghandle = xmmsc_mainloop_gmain_init(connection); // <-> No xmmsc_result_wait(res) 's.

  //cmd_volume(connection,100);
  //cmd_volume(connection,-1);
  return (1);
}

void close_xmms_session (void) {
  if(!connection) return;
  xmmsc_disconnect_callback_set (connection, NULL, NULL); 
  //xmmsc_io_disconnect(connection);
  // FIXME: stop callbacks!
  //xmmsc_mainloop_gmain_shutdown(connection, ghandle); 
  xmmsc_unref (connection);
  connection=NULL;
}

void xmms2_cmd (int cmd) {
  if (!connection) return;
  xmmsc_result_t *res;

  switch (cmd) {
    case XCMD_PLAY:
      res = xmmsc_playback_start (connection); break;
    case XCMD_PAUSE:
      res = xmmsc_playback_pause (connection); break;
    case XCMD_STOP:
      res = xmmsc_playback_stop (connection); break;
    case XCMD_NEXT:
      res = xmmsc_playlist_set_next_rel (connection, 1);
      //xmmsc_result_wait(res);
      XMMS2CERR(res)
      xmmsc_result_unref (res);
      res = xmmsc_playback_tickle (connection);
      break;
    case XCMD_CURR:
      res = xmmsc_playlist_set_next_rel (connection, 0);
      //xmmsc_result_wait(res);
      XMMS2CERR(res)
      xmmsc_result_unref (res);
      res = xmmsc_playback_tickle (connection);
      break;
    case XCMD_PREV:
      res = xmmsc_playlist_set_next_rel (connection, -1);
      //xmmsc_result_wait(res);
      XMMS2CERR(res)
      xmmsc_result_unref (res);
      res = xmmsc_playback_tickle (connection);
      break;
    default:
      fprintf(stderr, "unknown xmms2 command\n");
      return;
  }
  //xmmsc_result_wait(res);
  XMMS2CERR(res)
  xmmsc_result_unref (res);
}


void select_now_playing(void) {
  int idx;
  if (!connection) return;
  if(build_active || progress_update >=0)  return;
  if (strlen(curr_url)<1) return;
  printf("XMMS2: current song: %s\n",curr_url);

  idx=get_song_by_filename(curr_url);
  if (idx>=0) {
    search_select=TRUE;
    select_item("tv_artist",ART_IDX,song_db[idx]->Artist);
    select_item("tv_album",ALB_IDX,song_db[idx]->Album);
    select_item("tv_song",COL_IDX,idx);
    search_select=FALSE;
  }
}

void add_to_playlist (char *fn) {
  if (!connection) return;
  int ln=strlen(fn);
  if (ln>4 && (// match /^.*\.{pls|m3u}$/ 
    !(strncmp(".m3u",&(fn[ln-4]),4)) || !(strncmp(".pls",&(fn[ln-4]),4))
	      )) 
    add_playlist(connection, fn);
  else 
    add_item_to_playlist(connection, NULL, fn);
  //ml_lookup(fn); // XXX medialib - mp3b sync
}

int xmms_is_connected (void) { return(connection?1:0); }
#else 

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
/** No xmms2 */

int init_xmms_session (void) { ; }
void close_xmms_session (void) { ; }
void xmms2_cmd (int cmd) { ; }
void select_now_playing(void) { ; }
void add_to_playlist (char *fn) { ; }
int xmms_is_connected (void) { return(0); }

#endif

/* vi:set ts=8 sts=2 sw=2: */
